/* Copyright (c) 2014 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 *
 * This file contains the source code for a sample application using the Heart Rate service
 * (and also Battery and Device Information services). This application uses the
 * @ref srvlib_conn_params module.
 */

#include <string.h>
#include <stdio.h>
#include "pstorage.h"
#include "nordic_common.h"
#include "nrf.h"
#include "nrf_error.h"
#include "nrf_nvmc.h"
#include "app_error.h"
#include "nrf_soc.h"
#include "flash.h"

/* 
 *  Macro defines
 */
#define DATA_BLOCK_SIZE         32 
#define BLOCK_COUNT             2
#define INIT_SUCCESSFUL         1 
#define INIT_FAILED             0

/* 
 * Global variables  
 */
static pstorage_handle_t        m_storage_handle;
char                            m_buff[DATA_BLOCK_SIZE] __attribute__((aligned(4)));
static uint8_t                  m_init = INIT_FAILED;


/**@brief callback for storage
 *
 * @details handles the response from the storage module.
 */
static void storage_callback(pstorage_handle_t *handle, uint8_t op_code, uint32_t result, uint8_t *p_data, uint32_t len)
{

    switch (op_code)
    {

        case PSTORAGE_STORE_OP_CODE:             
            break;

        case PSTORAGE_LOAD_OP_CODE:
            break;

        case PSTORAGE_CLEAR_OP_CODE:
            break;

        case PSTORAGE_UPDATE_OP_CODE:
            break;

        default:
            break;
    }
}



/**@brief Function to store the device name in pstorage 
 *
 * @details Formats the packet with device name and puts the data
 *          into persistent storage  
 */
void store_device_name(char *name)
{
    uint8_t len; 
    pstorage_handle_t handle;

    if (INIT_SUCCESSFUL == m_init)
    {
        len = (uint8_t)strlen(name);
        strcpy(m_buff, name);

        // write the given device name 
        pstorage_block_identifier_get(&m_storage_handle, 0, &handle);
        pstorage_clear(&handle, DATA_BLOCK_SIZE);
        uint32_t err_code = pstorage_store(&handle, (uint8_t *)m_buff, len, 0);
        APP_ERROR_CHECK(err_code);
    }
}



/**@brief Function to get a valid device name 
 *
 * @details Returns the stored valid name or the default otherwise 
 */
char *get_device_name(void)
{
    uint32_t err_code;
    pstorage_handle_t  handle;

    if (INIT_SUCCESSFUL == m_init)
    {
        pstorage_block_identifier_get(&m_storage_handle, 0, &handle);
        err_code = pstorage_load((uint8_t *)m_buff, &handle, MAX_DEVICE_NAME_LENGTH, 0);
        APP_ERROR_CHECK(err_code);
   }

    return m_buff;
}


/**@brief Function for pstorage initialization.
 *
 * @details Initializes the storage module.
 */
void flash_init(void)
{
    uint32_t err_code; 
    pstorage_module_param_t storage_params;

    storage_params.cb = storage_callback;
    storage_params.block_size = DATA_BLOCK_SIZE; 
    storage_params.block_count = BLOCK_COUNT;

    err_code = pstorage_init();
    if (NRF_SUCCESS == err_code)
    {
        err_code = pstorage_register(&storage_params, &m_storage_handle);
        if (NRF_SUCCESS == err_code)
            m_init = INIT_SUCCESSFUL;
    }
}


